"""Simple JDOM document from XNI parser prototype"""

from org.jdom import *
from org.apache.xerces import *
import sys
import java

# todo: error handlers

VALIDATION = "http://xml.org/sax/features/validation"

class XNIBuilder(parsers.XMLDocumentParser):

   def __init__(self,
                config = parsers.StandardParserConfiguration(),
                expandEntities = 1,
                validate = 0,
                processIgnorableWhitespace = 0):
      parsers.XMLDocumentParser.__init__(self, config)
      self.declaredNamespaces = []
      self.availableNamespaces = [ Namespace.XML_NAMESPACE ]
      config.setFeature(VALIDATION, validate)
      self.expandEntities = expandEntities
      self.documentInfo = DocumentInfo()
      self.processIgnorableWhitespace = processIgnorableWhitespace

   def startDocument(self, locator, encoding):
      self.document=Document(None)
      self.parents = []
      self.inCData = 0
      self.inEntity = 0

   def xmlDecl(self, version, encoding, standalone):
      self.documentInfo.version = version
      self.documentInfo.encoding = encoding
      self.documentInfo.standalone = standalone

   def doctypeDecl(self, rootElement, publicId, systemId):
      self.docType = DocType(rootElement, publicId, systemId)

   def startPrefixMapping(self, prefix, uri):
      self.declaredNamespaces.append(Namespace.getNamespace(prefix, uri))

   def endPrefixMapping(self, prefix):
      self.availableNamespaces = [
         i for i in self.availableNamespaces if i.prefix != prefix]

   def startElement(self, element, attributes):
      el = self.__processElement(element, attributes)
      self.parents.append(el)

   def endElement(self, element):
      self.__removePrefixes(self.parents.pop())

   def emptyElement(self, element, attributes):
      self.__removePrefixes(self.__processElement(element, attributes))
      
   def __processElement(self, element, attributes):
      jdElement = Element(element.localpart,
                          Namespace.getNamespace(element.prefix, element.uri))

      if self.declaredNamespaces:
         self.__transferNamespaces(jdElement)

      n=xni.QName()
      for i in xrange(attributes.getLength()):
         attributes.getName(i,n)
         if n.prefix != "xmlns":
            jdElement.setAttribute(n.localpart,
                            attributes.getValue(i),
                            Namespace.getNamespace(n.prefix, n.uri))
      self.__add(jdElement)

      return jdElement

   def characters(self, text):
      if self.inCData:
         self.__add(CDATA(self.__asString(text)))
      elif self.expandEntities or not self.inEntity:
         self.__add(self.__asString(text))

   def ignorableWhitespace(self, text):
      if self.processIgnorableWhitespace:
         self.characters(text)

   def startCDATA(self):
      self.inCData = 1

   def endCDATA(self):
      self.inCData = 0

   def endDocument(self):
      pass

   def startEntity(self, name, publicId, systemId, baseSystemId, encoding):
      self.inEntity = 1

   def comment(self, text):
      self.__add(Comment(self.__asString(text)))

   def processingInstruction(self, target, data):
      self.__add(ProcessingInstruction(target, self.__asString(data)))

   def endEntity(self, name):
      self.inEntity = 0
      if name != '[dtd]' and self.expandEntities == 0:
         self.__add(EntityRef(name))

   def __removePrefixes(self, element):
      self.availableNamespaces = [
         n for n in self.availableNamespaces
         if not n in element.getAdditionalNamespaces() ]

   def __transferNamespaces(self, element):
      self.availableNamespaces.extend(self.declaredNamespaces)
      map(element.addNamespaceDeclaration, self.declaredNamespaces)
      self.declaredNamespaces = []

   def __add(self, content):
      if self.parents:
         self.parents[-1].addContent(content)
      else:
         self.document.setRootElement(content)

   def __asString(self, text):
      return java.lang.String(text.ch, text.offset, text.length)

class DocumentInfo:
   """Holder class for extra document information, for example the encoding"""
   pass

if __name__ == '__main__':
   try:
      parser = XNIBuilder(processIgnorableWhitespace = 1, validate = 1, expandEntities = 0)
      parser.parse(xni.parser.XMLInputSource(None, sys.argv[1], None))
      di = parser.documentInfo
      print "Document encoding was:", di.encoding
      output.XMLOutputter().output(parser.document, java.lang.System.out)

   except xni.XNIException, ex:
      print "yikes", ex, ex.exception


